@@ -364,6 +364,9 @@ module type S = sig
   module Topological : sig
     val fold : (V.t -> 'a -> 'a) -> t -> 'a -> 'a
     val iter : (V.t -> unit) -> t -> unit
+
+    val fold_stable : (V.t -> 'a -> 'a) -> t -> 'a -> 'a
+    val iter_stable : (V.t -> unit) -> t -> unit
   end
 
   val spanningtree : t -> E.t list
@@ -372,7 +375,7 @@ module type S = sig
   (** {2 Input / Output} *)
 
   val dot_output : t -> string -> unit
-    (** DOT output *)
+    (** DOT output in a file *)
 
   val display_with_gv : t -> unit
     (** Displays the given graph using the external tools "dot" and "gv"
@@ -381,6 +384,8 @@ module type S = sig
   val parse_gml_file : string -> t
   val parse_dot_file : string -> t
 
+  val print_gml : Format.formatter -> t -> unit
   val print_gml_file : t -> string -> unit
+  (* val print_graphml : Format.formatter -> t -> unit *)
 
 end
